const WORD = 'abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
const OTHERS = '1234567890_';

const parse = function(str) {
  let index = 0;
  const startStr = '{{_ .';
  const len = startStr.length;
  const matchDict = {
    0: 1,
    1: 2,
    2: 2,
    3: 3,
    4: 4
  };

  const is = function(chars, c) {
    return chars.indexOf(c) >= 0;
  };

  const match = function() {
    let count = 0;
    if (str.length - index < len) {
      return false;
    }
    for (var i = 0; i < len;i++) {
      if (startStr[i] === str[index + i]) {
        count++;
      } else {
        break;
      }
    }
    if (count === len) {
      return true;
    }
    index = index + matchDict[count];
    return match();
  };

  const char = function() {
    return str[index];
  };

  const next = function(c) {
    if (char() !== c) {
      throw new Error(`EXPECT ${c} but find ${char()}`);
    }
    index++;
  };

  const word = function() {
    const start = index;
    while (!isEnd() && is(WORD + OTHERS, char())) {
      index++;
    }
    return str.slice(start, index);
  };

  const key = function() {
    if (is(WORD, str[index])) {
      return {error: false, value: word()};
    } else {
      return {error: true, value: null};
    }
  };

  const value = function() {
    let value = '';
    let error = false;
    const start = index;
    while (char() !== '}' && !isEnd()) {
      index++;
    }
    const s = index === start ? value : str.slice(start, index);
    try {
      value = JSON.parse(s);
      next('}');
      next('}');
    } catch (e) {
      error = true;
    }
    return {error, value};
  };

  const isEnd = function() {
    return index >= str.length;
  };

  const space = function() {
    while ((str[index] === ' ') && !isEnd()) {
      index++;
    }
    return index;
  };

  const values = [];

  while (match()) {
    const start = index;
    index = index + len;
    const k = key();
    if (k.error) continue;
    space();
    const v = value();
    if (v.error) continue;
    const end = index;
    values.push({
      key: k.value,
      value: v.value,
      position: {start, end}
    });
  }
  return values;
};

const _replace = function(str, info, defaultValue) {
  let start = 0;
  const s = [];
  for (let i of info) {
    const p = str.slice(start, i.position.start);
    s.push(p);
    const value = i.key in defaultValue ? defaultValue[i.key] : i.value;
    s.push(value);
    start = i.position.end;
  }
  s.push(str.slice(start, str.length));
  return s.join('');
};

const replace = function(str, defaultValue = {}) {
  return _replace(str, parse(str), defaultValue);
};

module.exports = {
  replace,
  parse
};
